@@ -56,18 +56,18 @@ bool KConfigPrivate::mappingsRegistered = false;
 
 KConfigPrivate::KConfigPrivate(KConfig::OpenFlags flags,
                                QStandardPaths::StandardLocation resourceType)
-    : openFlags(flags), resourceType(resourceType), mBackend(0),
+    : openFlags(flags), resourceType(resourceType), mBackend(Q_NULLPTR),
       bDynamicBackend(true),  bDirty(false), bReadDefaults(false),
       bFileImmutable(false), bForceGlobal(false), bSuppressGlobal(false),
       configState(KConfigBase::NoAccess)
 {
     sGlobalFileName = QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation) + QLatin1String("/kdeglobals");
 
-    static int use_etc_kderc = -1;
-    if (use_etc_kderc < 0) {
-        use_etc_kderc = !qEnvironmentVariableIsSet("KDE_SKIP_KDERC");    // for unit tests
+    static QBasicAtomicInt use_etc_kderc = Q_BASIC_ATOMIC_INITIALIZER(-1);
+    if (use_etc_kderc.load() < 0) {
+        use_etc_kderc.store( !qEnvironmentVariableIsSet("KDE_SKIP_KDERC"));    // for unit tests
     }
-    if (use_etc_kderc) {
+    if (use_etc_kderc.load()) {
 
         etc_kderc =
 #ifdef Q_OS_WIN
@@ -95,7 +95,7 @@ KConfigPrivate::KConfigPrivate(KConfig::OpenFlags flags,
 //        mappingsRegistered = true;
 //    }
 
-    setLocale(QLocale::system().name());
+    setLocale(QLocale().name());
 }
 
 bool KConfigPrivate::lockLocal()
@@ -302,16 +302,22 @@ QStringList KConfigPrivate::groupList(const QByteArray &group) const
     return groups.toList();
 }
 
+static bool isGroupOrSubGroupMatch(const QByteArray &potentialGroup, const QByteArray &group)
+{
+    if (!potentialGroup.startsWith(group)) {
+      return false;
+    }
+    return potentialGroup.length() == group.length() || potentialGroup[group.length()] == '\x1d';
+}
+
 // List all sub groups, including subsubgroups
 QSet<QByteArray> KConfigPrivate::allSubGroups(const QByteArray &parentGroup) const
 {
     QSet<QByteArray> groups;
-    QByteArray theGroup = parentGroup + '\x1d';
-    groups << parentGroup;
 
     for (KEntryMap::const_iterator entryMapIt = entryMap.begin(); entryMapIt != entryMap.end(); ++entryMapIt) {
         const KEntryKey &key = entryMapIt.key();
-        if (key.mKey.isNull() && key.mGroup.startsWith(theGroup)) {
+        if (key.mKey.isNull() && isGroupOrSubGroupMatch(key.mGroup, parentGroup)) {
             groups << key.mGroup;
         }
     }
@@ -320,12 +326,10 @@ QSet<QByteArray> KConfigPrivate::allSubGroups(const QByteArray &parentGroup) con
 
 bool KConfigPrivate::hasNonDeletedEntries(const QByteArray &group) const
 {
-    const QSet<QByteArray> allGroups = allSubGroups(group);
-
-    Q_FOREACH (const QByteArray &aGroup, allGroups) {
-        // Could be optimized, let's use the slow way for now
+    for (KEntryMap::const_iterator it = entryMap.begin(); it != entryMap.end(); ++it) {
+        const KEntryKey &key = it.key();
         // Check for any non-deleted entry
-        if (!keyListImpl(aGroup).isEmpty()) {
+        if (isGroupOrSubGroupMatch(key.mGroup, group) && !key.mKey.isNull() && !it->bDeleted) {
             return true;
         }
     }
@@ -344,7 +348,7 @@ QStringList KConfigPrivate::keyListImpl(const QByteArray &theGroup) const
         QSet<QString> tmp;
         for (; it != theEnd && it.key().mGroup == theGroup; ++it) {
             const KEntryKey &key = it.key();
-            if (key.mGroup == theGroup && !key.mKey.isNull() && !it->bDeleted) {
+            if (!key.mKey.isNull() && !it->bDeleted) {
                 tmp << QString::fromUtf8(key.mKey);
             }
         }
@@ -513,6 +517,13 @@ QString KConfig::name() const
     return d->fileName;
 }
 
+
+KConfig::OpenFlags KConfig::openFlags() const
+{
+    Q_D(const KConfig);
+    return d->openFlags;
+}
+
 Q_GLOBAL_STATIC(QString, globalMainConfigName)
 
 void KConfig::setMainConfigName(const QString &str)
